const Main = imports.ui.main;
const OsdWindow = imports.ui.osdWindow;
const OsdWindowManager = Main.osdWindowManager;

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;
const GObject = imports.gi.GObject;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();


function init() {
  ExtensionUtils.initTranslations();
}


function style(color, bgcolor, alpha, shadow, border, rotate, monitorIndex) {

  let red = parseInt(parseFloat(color[0]) * 255);
  let green = parseInt(parseFloat(color[1]) * 255);
  let blue = parseInt(parseFloat(color[2]) * 255);
  
  let bgred = parseInt(parseFloat(bgcolor[0]) * 255);
  let bggreen = parseInt(parseFloat(bgcolor[1]) * 255);
  let bgblue = parseInt(parseFloat(bgcolor[2]) * 255);

  alpha = parseFloat(alpha/100.0);

  let hbx = OsdWindowManager._osdWindows[monitorIndex]._hbox;
  let ll = hbx.find_child_by_name('levLabel');

  let sty = `background-color: rgba(${bgred},${bggreen},${bgblue},${alpha}); color: rgb(${red},${green},${blue});`; 
  if (!shadow) sty += ' box-shadow: none;';
  if (border) sty += ` border-color: rgb(${red},${green},${blue});`;
  
  let levelsty = `-barlevel-active-background-color: rgb(${red},${green},${blue}); background-color: rgba(${red},${green},${blue},0.1);`;

  hbx.add_style_class_name(
    "osd-style"
  );
  hbx.style = sty;

  OsdWindowManager._osdWindows[monitorIndex]._label.style = ` font-size: ${10 + osd_size*0.5}px;`;
  if (ll != null) ll.style = ` font-size: ${8 + osd_size*0.75}px; font-weight: bold;`;

  if (rotate){
    hbx.set_pivot_point(0.0,1.0);
    hbx.rotation_angle_z = -90.0;
    
    if (ll != null) {
      ll.set_pivot_point(0.5,0.5);  
      ll.rotation_angle_z = 90.0;
    }
  }
  else {
    hbx.set_pivot_point(0.0,1.0);
    hbx.rotation_angle_z = 0;

    if (ll != null) {
      ll.set_pivot_point(0.5,0.5);
      ll.rotation_angle_z = 0.0;
    }
  }
  OsdWindowManager._osdWindows[monitorIndex]._level.style = levelsty;

}


function unCustom() {

  let hbx;
  for (
    let monitorIndex = 0;
    monitorIndex < OsdWindowManager._osdWindows.length;
    monitorIndex++
  ) {
    hbx = OsdWindowManager._osdWindows[monitorIndex]._hbox;
    hbx.remove_style_class_name(
      "osd-style"
    );
    hbx.style = '';
    hbx.rotation_angle_z = 0;
    hbx.set_pivot_point(0.0,0.0);
    ll = hbx.find_child_by_name('levLabel');
    if (ll != null) hbx.remove_child(ll);
    hbx.translation_x = 0;
    hbx.translation_y = 0;
    hbx.visible = true;

    OsdWindowManager._osdWindows[monitorIndex]._label.style = '';
    OsdWindowManager._osdWindows[monitorIndex]._level.style = '';
    OsdWindowManager._osdWindows[monitorIndex]._icon.icon_size = _size;
  }
}


function injectToFunction(parent, name, func) {
  let origin = parent[name];
  parent[name] = function () {
    let ret;
    ret = origin.apply(this, arguments);
    if (ret === undefined) ret = func.apply(this, arguments);
    return ret;
  };
  return origin;
}

function removeInjection(object, injection, name) {
  if (injection[name] === undefined) delete object[name];
  else object[name] = injection[name];
}

let injections = [];
let first_run = true;
let _size, osd_size ;
let _monIds = [];


function enable() {

  let _settings = ExtensionUtils.getSettings(
    "org.gnome.shell.extensions.custom-osd"
  );

  injections["show"] = injectToFunction(
    OsdWindow.OsdWindow.prototype,
    "show",
    function () {

      try{
      let monitor = Main.layoutManager.monitors[this._monitorIndex];
      let monitors = _settings.get_string("monitors");

      if (monitors == "primary" && monitor != Main.layoutManager.primaryMonitor){
        this._hbox.visible = false;
        return;
      }
      else if (monitors == "external" && monitor == Main.layoutManager.primaryMonitor){
        this._hbox.visible = false;
        return;
      }
      this._hbox.visible = true;

      osd_size = _settings.get_int("size");
      let h_percent = _settings.get_int("horizontal");
      let v_percent = _settings.get_int("vertical");
      let hide_delay = _settings.get_int("delay");
      let color = _settings.get_strv("color");
      let bgcolor = _settings.get_strv("bgcolor");
      let alpha = _settings.get_int("alpha");
      let shadow = _settings.get_boolean("shadow");
      let border = _settings.get_boolean("border");
      let rotate = _settings.get_boolean("rotate");
      let l_label = _settings.get_boolean("label");

      OsdWindow.HIDE_TIMEOUT = hide_delay;

      if (l_label) {
        if(!_monIds.includes(this._monitorIndex)){
        
          this._levLabel = new St.Label({
            name: 'levLabel',
            x_align: Clutter.ActorAlign.CENTER,
            y_align: Clutter.ActorAlign.CENTER,
          });
          this._level.bind_property_full(
            'value',
            this._levLabel,
            'text',
            GObject.BindingFlags.SYNC_CREATE,
            (__, v) => [true, (v * 100).toFixed()],
            null
          );
          this._hbox.insert_child_above(this._levLabel, this._vbox);
          _monIds.push(this._monitorIndex);
        }
        this._hbox.find_child_by_name('levLabel').visible = this._level.visible;
      }
      else {
        ll = this._hbox.find_child_by_name('levLabel');
        if (ll != null) ll.visible = false;
      }

      style(color, bgcolor, alpha, shadow, border, rotate, this._monitorIndex);

      this._hbox.translation_x = (h_percent * monitor.width) / 100 / 2;
      this._hbox.translation_y = (v_percent * monitor.height) / 100 / 2;

      if (first_run) {
        _size = this._icon.icon_size;
        first_run = false;
      }     
      this._icon.icon_size = 18 + (osd_size * monitor.height) / 100 / 7 ;

    }
    catch (e) {
      logError(e, 'ExtensionError');
    };

    }
  );

}

function disable() {

  unCustom();
  _monIds = [];
  OsdWindow.HIDE_TIMEOUT = 1500;
  
  removeInjection(OsdWindow.OsdWindow.prototype, injections, "show");
  
}
